Exception: SeccompTools::Error

Inherits:
StandardError
  • Object
show all
Defined in:
lib/seccomp-tools/error.rb

Overview

Base class of all errors raised by this library.

Rescue this class to catch every assembler error at once.