Exception: SeccompTools::Error
- Inherits:
-
StandardError
- Object
- StandardError
- SeccompTools::Error
- Defined in:
- lib/seccomp-tools/error.rb
Overview
Base class of all errors raised by this library.
Rescue this class to catch every assembler error at once.
Direct Known Subclasses
BackwardJumpError, DuplicateLabelError, LongJumpError, ParseError, UndefinedLabelError, UnrecognizedTokenError